;;; -*- Mode: LISP; Syntax: Common-lisp; Package: QPE -*-

(in-package 'QPE)

(defvar *axiom-indicators* '(rassert! rjustify rnogood rassume nogood justify assume assert!
				      *QC-Qstate* install-derived-qstate-pairs
				      adb::add-to-nested-alist))


(defun axioms-in-form (form)
  (cond ((null form) 0)
	((symbolp form)
	 (if (member form *axiom-indicators*) 1 0))
	((not (listp form)) 0)
	(t (+ (axioms-in-form (car form))
	      (axioms-in-form (cdr form))))))

(setq foo 'cl-user::(defProcess (Liquid-Flow-Thermal ?lf)
   Individuals ((?src :type Thermal-Contained-Liquid)
		(?dst :type Thermal-Contained-Liquid)
		(?lf :conditions ((process-instance liquid-flow) ?lf)
		                 (?lf src ?src)
		                 (?lf dst ?dst)))
   QuantityConditions ((Active ?lf)
		       (not (equal-to (A (Temperature ?src)) (A (Temperature ?dst)))))
   Relations ((Quantity Thermal-Mixing)	;want gas & liquid to coexist
	      (Q= Thermal-Mixing (- (Temperature ?src) (Temperature ?dst))))
   Influences ((I+ (Intensive-Energy ?dst) (A Thermal-Mixing)))))

(defun axioms-in-file (from-file)
  (let ((acount 0) ;; axiom-equivalent count
	(pcount 0) ;; processes
	(vcount 0) ;; views
	(rcount 0) ;; rules
	(qcount 0) ;; quantities
	(percount 0) ;; perspectives
	(ecount 0) ;; entities
	(ocount 0) ;; operators
	(predcount 0) ;; predicates
    (marker (cons nil nil)))
  (with-open-file (in-box from-file :direction :input)
    (do ((form (read in-box nil marker) (read in-box nil marker)))
	((eq form marker) (values acount pcount vcount rcount qcount percount ecount ocount
				  predcount))
      ;; Update axiom count
      (incf acount (axioms-in-form (macroexpand form)))
      (when (listp form)
	(case (car form)
	  ;; Assume appropriate symbols have been imported into the QPE package.
	  (DEFPROCESS (incf pcount))
	  (DEFVIEW (incf vcount))
	  ((RULE adb::RULE srule adb::srule) (incf rcount))
	  (DEFQUANTITY-TYPE (incf qcount))
	  (DEFPERSPECTIVE (incf percount))
	  (DEFENTITY (incf ecount))
	  (DEFPREDICATE (incf predcount))
	  (DEFOPERATOR (incf ocount))
	  ((DEFVAR SETQ EVAL-WHEN DECLARE PROCLAIM IN-PACKAGE ADB::RULES-FILE
		   defcomputable-relation assertq))
	  (t (format t "~% AXIOMS-IN-FILE: Unknown form class in domain file: ~A"
		     (car form)))))))))

(defun tabulate-domain (dfiles &optional (stream *standard-output*))
  (let ((atotal 0)(ptotal 0)(vtotal 0)(rtotal 0)(qtotal 0)(pertotal 0)(etotal 0)
	(ototal 0)(predtotal 0))
    (dolist (dfile dfiles)
      (multiple-value-bind (acount pcount vcount rcount qcount
			    percount ecount ocount predcount)
	  (axioms-in-file dfile)
	(incf atotal acount)(incf ptotal pcount)(incf vtotal vcount)(incf rtotal rcount)
	(incf qtotal qcount)(incf pertotal percount)(incf etotal ecount) (incf ototal ocount)
	 (incf predtotal predcount)
	(say-domain-stats (format nil "For ~A:" dfile)
			  acount pcount vcount rcount qcount percount ecount ocount 
			  predcount stream)))
    (say-domain-stats "Overall:"
		      atotal ptotal vtotal rtotal qtotal 
		      pertotal etotal ototal predtotal stream)))

(defun say-domain-stats (header axioms processes views rules qtypes persps ents ocount
				preds
				&optional (stream *standard-output*))
  (format stream "~%~A:" header)
  (format stream "~% ~D quantity types, ~D entities, ~D predicates, ~D perspectives"
	  qtypes ents preds persps)
  (format stream "~% ~D views, ~D processes, ~D explicit rules, ~D operators."
	  views processes rules ocount)
  (format stream "~% ~D axiom-equivalents." axioms))
  

(setq *steam-plant-domain-files*
'( "heath:>falken>qp>domains>steam-plant>physob-cs"
 "heath:>falken>qp>domains>steam-plant>thermo-objects"
 "heath:>falken>qp>domains>steam-plant>sources-sinks"
 "heath:>falken>qp>domains>steam-plant>level1"
 "heath:>falken>qp>domains>steam-plant>level4"
 "heath:>falken>qp>domains>steam-plant>level5"
 "heath:>falken>qp>domains>steam-plant>operating-assumptions"))