;;; -*- Mode: LISP; Syntax: Common-lisp; Package: QPE -*-

;;;; Closing various QPE-related facts

(in-package 'QPE)

(defun close-quantities ()
  ;; Ensures that quantity assertions are closed.
  ;; Now happens as standard part of envisioning.
  (dolist (q (gather-quantities))
    (unless (eq (car q) 'net-influence)
      (adb::close-fact `(quantity ,q) :TRUE))))

(defun check-quantity-closure ()
  ;; For debugging purposes
  (setq *old-labels*
	(mapcar #'(lambda (q) (cons q (adb::environments `((quantity ,q) . :TRUE))))
		(gather-quantities)))
  (close-quantities)
  (setq *new-labels*
	(mapcar #'(lambda (q) (cons q (adb::environments `((quantity ,q) . :TRUE))))
		(gather-quantities)))
  (setq *diffs* nil)
  (dolist (oentry *old-labels*)
    (let ((nentry (assoc (car oentry) *new-labels* :Test #'equal)))
      (unless (equal (cdr oentry) (cdr nentry))
	(push (list (car oentry) (cdr oentry) (cdr nentry)) *diffs*))))
  (length *diffs*))


;;;; Defining relationships to close

;; Suppose one wanted to close all instances of a particular relationship,
;; such as CONNECTED-TO.  One can say this by 
;; (defClosed-Predicate <predicate> &optional <mod/op pair>)
;; This will cause all instances of <predicate> to be closed, at the
;; time specified by <mod/op pair>.  The time defaults to :AFTER :PSVS-ASNS.

(defvar *closed-predicates* nil)
(adb::init-form (setq *closed-predicates* nil))

(defmacro defClosed-Predicate (relation &rest options)
  (unless (or (null options)
	      (and (member (car options) '(:BEFORE :AFTER))
		   (member (cadr options) *internal-event-types*)
		   (or (null (cddr options))
		       (integerp (caddr options)))))
    (error "Unknown closing time specified for ~A: ~A" relation options))
  (let ((form `(progn 'COMPILE ;; superstitious?
		      (push ',relation *closed-predicates*) 
		      (schedule-qpe-event #'(lambda () (close-instances-of ',relation))
					  ,(if options (car options) ':AFTER)
					  ,(if options (cadr options) ':PSVS-ASNS)
					  ,(if (cddr options) (caddr options)
					       500))))) ;; leave lots of room.
    (adb::eval-or-packetize form)))

(defun close-instances-of (relation)
  (dolist (fact (adb::fetch `((,relation . ?x) . :TRUE)))
    (adb::close-fact (car fact) :TRUE)))
