;; -*- Mode: Lisp; Syntax: Common-lisp; Package: (QPE :use (Common-Lisp ADB)) -*- 

;;; The Qualitative Process theory Engine
;;
;; Uses the ATMoSphere inference engine to provide a (hopefully
;; very efficient) simulator for QP theory.

;; This system definition covers the core engine.  It is designed 
;; to be very portable -- should work with any reasonably complete
;; implementation of Common Lisp.
;;
;; To do this means eschewing graphics in the core.  Consequently, the
;; default user-interface is character-oriented.  Graphical interfaces
;; are provided for a few implementations, but please feel free to write
;; your own.

#+Symbolics
(in-package 'QPE :use '(Common-Lisp ADB))
#-Symbolics
(in-package 'QPE :use '(lisp ADB))

;; In this Post-Symbolics world, we revert to
;; the awful days of the mid-70's.  SIGH!

;; These two get around some non-standard brain-damage
;;  in the Symbolics world.
(defvar *user-package*
	#+:Symbolics (find-package 'cl-user)
	#-:Symbolics 'user)

(defvar *lisp-package*
	#+:Symbolics (find-package 'common-lisp)
	#-:Symbolics 'lisp)

(defvar *version-number* "2.6 Beta")

;;;;==============================================================================
;;;  THESE VARIABLES NEED TO BE UPDATED WHEN INSTALLING QPE AT A NEW SITE.

;; The specializations listed here are for my site.  You may want to install
;; your own conditionalizations by adding to *features*.

;; WARNING: The flags RT, Symbolics indicate machine type.  I suggest that you
;; do NOT fool with these flags.  Instead, define your own site-indicator and
;; push it onto *features*.

;; Location of sources
(defvar *qpe-pathname*
	#+QRG-RT "/u/forbus/code/qpe/v2-6/"
	#+KDF-RT "/u/qpe/v2-6/"
	#+(and UIUC SYMBOLICS-RELEASE-7) "DOC:>QRG>qp>qpe>"
	#+(and UIUC SYMBOLICS-RELEASE-6) "rube:>qp>qpe>v2-6>"
	#+JOHAN "calvin:>qp>qpe>cl>"
	#+PARC "bd:>qp>qpe>cl>"
	)

;; These are for interface hackery.
(defvar *domain-path*
	#+QRG-RT "/u/forbus/code/qpe/domains/"
	#+KDF-RT "/u/qpe/domains/"
	#+(and UIUC SYMBOLICS-RELEASE-7) "DOC:>QRG>qp>domains>"
	#+(and UIUC SYMBOLICS-RELEASE-6) "rube:>qp>domains>"
	#+JOHAN "calvin:>qp>domains>cl>"
	#+PARC "bd:>qp>domains>cl>"
	)					

(defvar *examples-path*
	#+QRG-RT "/u/forbus/code/qpe/examples/"
	#+KDF-RT "/u/qpe/examples/"
	#+(and UIUC SYMBOLICS-RELEASE-7) "DOC:>QRG>qp>examples>"
	#+(and UIUC SYMBOLICS-RELEASE-6) "rube:>qp>examples>"
	#+JOHAN "calvin:>qp>examples>cl>"
	#+PARC "bd:>qp>examples>cl>"
	)

(setq *QPE-rules-files*
  #+KDF-RT '("/u/qpe/v2-6/qrules.bbin")
  #+QRG-RT '("/u/forbus/code/qpe/qrules.bbin")
  #+(and UIUC SYMBOLICS-RELEASE-7) '("DOC:>QRG>qp>qpe>qrules.bin")
  #+(and UIUC SYMBOLICS-RELEASE-6) '("rube:>qp>qpe>v2-6>qrules.bin")
  #+JOHAN '("calvin:>qp>qpe>cl>qrules.bin")
  #+PARC '("bd:>qp>qpe>cl>qrules.bin")
  )

;;; More installation-specific constants
;; These defaults should be changed on installation.

(defvar *domain-files*
  #+QRG-RT '("/u/forbus/code/qpe/domains/nst.lisp")
  #+KDF-RT '("/u/qpe/domains/nst.lisp")
  #+(and UIUC SYMBOLICS-RELEASE-7) '("DOC:>QRG>qp>domains>nst.lisp>")
  #+(and UIUC SYMBOLICS-RELEASE-6) '("rube:>qp>domains>nst.lisp")
  #+JOHAN '("calvin:>qp>domains>cl>nst.bin")
  #+PARC '("bd:>qp>domains>cl>nst.bin")
  )

(defvar *scenario-files*
  #+(and UIUC SYMBOLICS-RELEASE-7) '("DOC:>QRG>qp>examples>ex1.lisp>")
  #+(and UIUC SYMBOLICS-RELEASE-6) '("rube:>qp>examples>ex1.lisp")
  #+KDF-RT '("/u/qpe/examples/ex1.lisp")
  #+QRG-RT '("/u/forbus/code/qpe/examples/ex1.lisp")
  #+JOHAN '("calvin:>qp>examples>cl>ex1.lisp")
  #+PARC '("bd:>qp>examples>cl>ex1.lisp")
  )

(defvar *benchmark-physics-files*
  #+UIUC '("rube:>qp>domains>NST.lisp")
  #+QRG-RT '("/u/forbus/code/qpe/domains/nst.lisp")
  #+KDF-RT '("/u/qpe/domains/nst.lisp")
  #+JOHAN '("calvin:>qp>domains>cl>NST.bin")
  #+PARC '("bd:>qp>domains>cl>NST.bin")
  )

(defvar *QPE-benchmark-data-directory*
  #+UIUC "rube:>qp>runs>"
  #+KDF-RT "/u/qpe/runs/"
  #+QRG-RT "/u/forbus/code/qpe/runs/"
  #+JOHAN "calvin:>qp>runs>"
  #+PARC "bd:>qp>runs>"
)

;;; Examples for testing/benchmarking

;; Please feel free to add your own.
 
(defvar *benchmark-scenarios* '(("Example 1 -- Two containers"
				  "ex1" #+UIUC "rube:>qp>examples>ex1.lisp"
				  #+JOHAN  "calvin:>qp>examples>cl>ex1.lisp"
				  #+PARC  "bd:>qp>examples>cl>ex1.lisp"
				  #+KDF-RT "/u/qpe/examples/ex1.lisp"
				  #+QRG-RT "/u/forbus/code/qpe/examples/ex1.lisp")
				 ("Example 2 -- Boiling water"
				  "ex2" #+UIUC "rube:>qp>examples>ex2.lisp"
				  #+JOHAN "calvin:>qp>examples>cl>ex2.lisp"
				  #+PARC "bd:>qp>examples>cl>ex2.lisp"
				  #+KDF-RT "/u/qpe/examples/ex2.lisp"
				  #+QRG-RT "/u/forbus/code/qpe/examples/ex2.lisp")
				 ("Example 3 -- Three containers"
				  "ex3" #+UIUC "rube:>qp>examples>ex3.lisp"
				   #+JOHAN "calvin:>qp>examples>cl>ex3.lisp"
				   #+PARC "bd:>qp>examples>cl>ex3.lisp"
				   #+KDF-RT "/u/qpe/examples/ex3.lisp"
				   #+QRG-RT "/u/forbus/code/qpe/examples/ex3.lisp")
				 ("Example 4 -- Double heat flow"
				  "ex4" #+UIUC "rube:>qp>examples>ex4.lisp"
				   #+JOHAN "calvin:>qp>examples>cl>ex4.lisp"
				   #+PARC "bd:>qp>examples>cl>ex4.lisp"
				   #+KDF-RT "/u/qpe/examples/ex4.lisp"
				   #+QRG-RT "/u/forbus/code/qpe/examples/ex4.lisp")
				 ("Example 5 -- Three blobs"
				  "ex5" #+UIUC "rube:>qp>examples>ex5.lisp"
				  #+JOHAN "calvin:>qp>examples>cl>ex5.lisp"
				  #+PARC "bd:>qp>examples>cl>ex5.lisp"
				   #+RT "/u/qpe/examples/ex5.lisp"
				   #+QRG-RT "/u/forbus/code/qpe/examples/ex5.lisp")
				 ("Example 6 -- Four blobs"
				  "ex6" #+UIUC "rube:>qp>examples>ex6.lisp"
				  #+JOHAN "calvin:>qp>examples>cl>ex6.lisp"
				  #+PARC "bd:>qp>examples>cl>ex6.lisp"
				   #+KDF-RT "/u/qpe/examples/ex6.lisp"
				   #+QRG-RT "/u/forbus/code/qpe/examples/ex6.lisp")
				 ("Example 7 -- Four containers"
				  "ex7" #+UIUC "rube:>qp>examples>ex7.lisp"
				   #+JOHAN "calvin:>qp>examples>cl>ex7.lisp"
				   #+PARC "bd:>qp>examples>cl>ex7.lisp"
				   #+KDF-RT "/u/qpe/examples/ex7.lisp"
				   #+QRG-RT "/u/forbus/code/qpe/examples/ex7.lisp")))

;;;;==============================================================================

;;;; QPE system definition

(pcl::defsystem QPE *qpe-pathname*
 ;; <name> <preload for loading> <preload for compiling> <recompile if changed>
 ((symbols () () ())                    ;; Package hackery -- Must be loaded 1st!!
  (defs (symbols) (symbols) (symbols))  ;; Structure definitions
  (close (defs) (defs) (defs))
  (events (symbols) (symbols) (symbols))
  (cmenu () () ())                      ;; Character-oriented menu system
  (careful () (defs) ())                ;; Careful mode
  (timer () () ())                      ;; Simple timing facility
  (qdefs (defs) (defs symbols careful)  ;; Basic quantity definitions
	  (defs))
  (trans (qdefs) (qdefs) (qdefs))       ;; Transitivity system
  (corres (qdefs) (qdefs) (qdefs))      ;; Implementing correspondences
  (efunct (qdefs) (qdefs) (qdefs))      ;; Implementing explicit functions
  (qrules (qdefs) (qdefs) (qdefs))	;; Instantiation rules
  (cdi (defs) (defs) (defs))		;; Objects, Process, and Individual View macros
  (persp (cdi) (cdi) (cdi))
  (syntaxer (qdefs) (qdefs)(qdefs))     ;; QPE syntaxer -- data driven, extendable. 
  (syntax (syntaxer) (syntaxer) (syntaxer)) ;; Default syntax
  (pcvary (syntaxer) (syntaxer symbols) ;; Manipulating preconditions
	  (syntaxer))
  (psvs (pcvary close) (pcvary close) (pcvary close))   ;; Constructing PS/VS solutions
  (resolve (psvs cdi) (psvs cdi) (psvs cdi));; Influence resolution
  (limit (resolve) (resolve) (resolve))     ;; Limit analysis
  (top (limit careful timer events)                ;; Toplevel
       (limit careful timer events)
       (pcvary events))             
  (bench (top) (top) (top))
  (debug (top) (top) (top))                 ;; Debugging code
  (qlprint (debug) (debug) (qdefs))         ;; Hook for lprint
  (qprint (qlprint) (qlprint) (qdefs qlprint))    ;; Report generator
  (chartop (qprint cmenu top) (qprint cmenu top)      ;; Character-oriented interface.
	    (qdefs cmenu top))
  ))

;;;; Functions for loading, compiling, etc.

;; Plus some random patches

(defvar *adb-loaded* nil)

(defun load-adb-if-needed ()
  (unless *adb-loaded* 
	  (adb::load-adb)
	  (setq *adb-loaded* t)
	  (adb::init t)))

(defun load-qpe ()
  (load-adb-if-needed)
  (format t "~%Loading QPE (version ~A)" *version-number*)
  (pcl::operate-on-system 'qpe :Load))

(defun compile-qpe ()
  (load-adb-if-needed)
  (pcl::operate-on-system 'qpe :compile))

(defun recompile-qpe ()
  (load-adb-if-needed)
  (adb::init)
  (pcl::operate-on-system 'qpe :recompile))

;; Need to get the right internals from Lucid
#+Lucid (defun qpe::gc () (user::gc))
#+Lucid (defun qpe::gc-on () (format t "~%I sure hope the GC is on..."))
#+Lucid (defun qpe::gc-off () (format t "~%I sure hope the GC is off..."))
