;;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: QPE; -*-

(in-package 'QPE)

;;;; Explicit functions
;; These statement take the form (function-spec <identifier> <list of Qprops>)
;; The list of Qprops is assumed to be complete.
;; The import of these statements is that between any two objects which have this
;; explicit function, there is a correspondence between their values.

(defun parse-explicit-function (spec &aux value ind arguments)
  (when (not (symbolp (car spec)))
    (error "Explicit function names must be symbols -- ~A." (car spec)))
  (setq value (cadr (cadr spec))
	ind (cadr value))
  ;Take the first arg to first Qprop is the quantity set by the function
  (dolist (qprop (cdr spec))
    ;; Check and accumulate arguments
    (unless (member (car qprop) '(Qprop Qprop-))
      (error "Only QPROP's may appear in a FUNCTION-SPEC -- ~A." qprop))
    (unless (and (equal (cadr qprop) value)
		 (equal (cadr (caddr qprop)) ind))
      (error "Qualitative proportionality must constrain ~A -- ~A." value qprop))
    (push (caddr qprop) arguments))
  (values `((has-function ,ind ,(car spec)) . :TRUE) (car spec) value arguments))

(defun install-explicit-function-specs (fun value arguments)
  (setf (get fun 'Explicit-function-specs)  (cons value arguments)))

(defun get-explicit-function-specs (fun &aux specs)
  (setq specs (get fun 'Explicit-function-specs))
  (unless specs
    (error"~A not an EXPLICIT-FUNCTION identifier..." fun))
  (values (car specs) (cdr specs)))

(defun install-explicit-function (obj1 obj2 fun &aux obj-var)
  (multiple-value-bind (value arguments)
      (get-explicit-function-specs fun)
    (setq obj-var (cadr value)) ;; get var to subst into
    (adb:justify `((rcorrespondence ((A ,(subst obj1 obj-var value :test #'equal))
				     (A ,(subst obj2 obj-var value :test #'equal)))
				    ,@ (mapcar #'(lambda (argument)
						   `((A , (subst obj1 obj-var argument
								 :test #'equal))
						     (A , (subst obj2 obj-var argument
								 :test #'equal))))
					       arguments)) . :TRUE)
		  `(((has-function ,obj1 ,fun) . :TRUE)
		   ((has-function ,obj2 ,fun) . :TRUE)))))

