;;; -*- Syntax: Common-lisp; Mode: LISP; Package: QPE -*-

(in-package 'QPE)

;;;; Propagating inequalities across algebraic operators

(adb::rules-file "Q=propagation")

;; These were originally developed by Brian Falkenhainer.

(adb::rule :intern
	   ((Equal-to (A (?qt . ?qargs1)) (A (?qt . ?qargs2)))
	    :test (not (equal ?qargs1 ?qargs2)) :var ?eq-form ;; more tests?
	    (Q= (?qt . ?qargs1) (?fun ?q1 ?q2)) :test (member ?f '(+ *+)) :var ?F1
	    (Q= (?qt . ?qargs2) (?fun ?q3 ?q4)) :var ?f2)
  (nlet ((?gt-form ((Greater-Than (A (?qt . ?qargs1)) (A (?qt . ?qargs2))) . :TRUE))
	 (?lt-form ((Less-Than (A (?qt . ?qargs1)) (A (?qt . ?qargs2))) . :TRUE))
	 (?deq-form  ((Equal-To (D (?qt . ?qargs1)) (D (?qt . ?qargs2))) . :TRUE)))
	(let ((1<3 (qpe::make-qrel-assertion (list 'A ?q1) (list 'A ?q3) 'Less-than))
	      (1=3 (qpe::make-qrel-assertion (list 'A ?q1) (list 'A ?q3) 'equal-to))
	      (D1=3 (qpe::make-qrel-assertion (list 'D ?q1) (list 'D ?q3) 'equal-to))
	      (1>3 (qpe::make-qrel-assertion (list 'A ?q1) (list 'A ?q3) 'Greater-than))
	      (2<4 (qpe::make-qrel-assertion (list 'A ?q2) (list 'A ?q4) 'Less-than))
	      (2=4 (qpe::make-qrel-assertion (list 'A ?q2) (list 'A ?q4) 'Less-than))
	      (D2=4 (qpe::make-qrel-assertion (list 'A ?q2) (list 'A ?q4) 'Less-than))
	      (2>4 (qpe::make-qrel-assertion (list 'A ?q2) (list 'A ?q4) 'Less-than)))
	  (cond ((and (equal ?q1 ?q3) (equal ?q2 ?q4)) ;; RHS' are equal
		 (adb::justify ?eq-form (list ?f1 ?f2) :RHS-IDENTICAL)
		 (adb::justify ?deq-form (list ?f1 ?f2) :RHS-IDENTICAL))
		((equal ?q1 ?q3) ;; First term same, so result fixed by second.
		 (adb::justify ?eq-form (list ?f1 ?f2 2=4) :2nd-term-equal)
		 (adb::justify ?deq-form (list ?f1 ?f2 D2=4) :2nd-term-equal)
		 (adb::justify ?gt-form (list ?f1 ?f2 2>4) :2nd-term-greater)
		 (adb::justify ?lt-form (list ?f1 ?f2 2<4) :2nd-term-smaller))
		((equal ?q2 ?q4) ;; Second term same, so result fixed by first
		 (adb::justify ?eq-form (list ?f1 ?f2 1=3) :1st-term-equal)
		 (adb::justify ?deq-form (list ?f1 ?f2 D1=3) :1st-term-equal)
		 (adb::justify ?gt-form (list ?f1 ?f2 1>3) :1st-term-greater)
		 (adb::justify ?lt-form (list ?f1 ?f2 1<3) :1st-term-smaller))
		(t ;; Now install cases that depend on inequalities involving both terms.
		 (adb::justify ?eq-form (list ?f1 ?f2 1=3 2=4) :propagate-equality)
		 (adb::justify ?deq-form (list ?f1 ?f2 d1=3 d2=4) :propagate-equality)
		 (adb::justify ?gt-form (list ?f1 ?f2 1>3 (qpe::flip-truths 2<4))
			       :propagate-inequality)
		 (adb::justify ?gt-form (list ?f1 ?f2 (qpe::flip-truths 1<3) 2>4)
			       :propagate-inequality)
		 (adb::justify ?lt-form (list ?f1 ?f2 1<3 (qpe::flip-truths 2>4))
			       :propagate-inequality)
		 (adb::justify ?lt-form (list ?f1 ?f2 (qpe::flip-truths 1>3) 2<4)
			       :propagate-inequality))))))