;;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: QPE; -*-

(in-package 'QPE)

;;;; QPE augments to LPRINT.

(adb::defForm-Parser (and . ?conjuncts)
 (cond ((null ?conjuncts) nil)
       ((cdr ?conjuncts)
	(nconc  (list :START :LP) 
		  (cdr (mapcan #'(lambda (conjunct)
				     (list (adb::get-and-symbol) 
					   (adb::parse-form conjunct)))
				 ?conjuncts))
		  (list :RP :END)))
       (t (adb::parse-form (car ?conjuncts)))))

(adb::defForm-Parser (or . ?disjuncts)
 (cond ((null ?disjuncts) nil)
       ((cdr ?disjuncts)
	(nconc (list :START :LP)
		(cdr (mapcan #'(lambda (disjunct)
				 (list (adb::get-or-symbol)
				       (adb::parse-form disjunct)))
			     ?disjuncts))
		(list :RP :END)))
       (t (adb::parse-form (car ?disjuncts)))))

(adb::defForm-Parser (+ . ?terms)
 (cond ((null ?terms) nil)
       ((cdr ?terms)
	(nconc (list :START :LP)
		(cdr (mapcan #'(lambda (term) (list "+" (adb::parse-form term)))
			     ?terms))
		(list :RP :END)))
       (t (adb::parse-form (car ?terms)))))

(adb::defForm-Parser (-> ?ante ?conse)
 (list :START :LP (adb::parse-form ?ante)
       (adb::get-imp-symbol) (adb::parse-form ?conse) :RP :END))

(adb::defForm-Parser (iff ?ante ?conse)
  (list :START :LP (adb::parse-form ?ante)
	(adb::get-eqv-symbol) (adb::parse-form ?conse) :RP :END))

(adb::defForm-Parser (not ?thing)
  (list :START :LP (adb::get-not-symbol) (adb::parse-form ?thing) :RP :END))

(adb::defForm-Parser (:not ?thing)
  (list :START :LP (adb::get-not-symbol) (adb::parse-form ?thing) :RP :END))

(adb::defForm-Parser (Greater-Than ?x ?y)
	(list :START (adb::parse-form ?x) (adb::get-gt-symbol) (adb::parse-form ?y) :END))

(adb::defForm-Parser (Less-Than ?x ?y)
    (list :START (adb::parse-form ?x) (adb::get-lt-symbol) (adb::parse-form ?y) :END))

(adb::defForm-Parser (Equal-To ?x ?y)
    (list :START (adb::parse-form ?x) (adb::get-eq-symbol) (adb::parse-form ?y) :END))

(adb::defForm-Parser (= ?x ?y)
    (list :START (adb::parse-form ?x) (adb::get-eq-symbol) (adb::parse-form ?y) :END))

(adb::defForm-Parser (LT-EQ ?x ?y)
	(list :START (adb::parse-form ?x) (adb::get-leq-symbol)
	      (adb::parse-form ?y) :END))

(adb::defForm-Parser (GT-EQ ?x ?y)
	(list :START (adb::parse-form ?x) (adb::get-geq-symbol)
	      (adb::parse-form ?y) :END))

(adb::defForm-Parser (unrelated ?x ?y)
    (list :START (adb::parse-form ?x) (adb::get-unrelated-symbol)
	  (adb::parse-form ?y) :END))

(adb::defForm-Parser (s ?x)
   (cond ((listp ?x)
	  (case (car ?x)
	    (D ;; Ds value
	      (list :START "Ds[" (adb::parse-form (cadr ?x)) "]" :END))
	    (A ;; As value
	      (list :START "As[" (adb::parse-form (cadr ?x)) "]" :END))
	    (t ;; Should be bogus, but, hey, we're nice
	      (list :START "s[" (adb::parse-form ?x) "]" :END))))
	 (t (list :START "s[" (adb::parse-form ?x) "]" :END))))

(adb::defForm-Function S)

(adb::defForm-Parser (m ?x)
   (cond ((listp ?x)
	  (case (car ?x)
	    (D ;; Dm value
	      (list :START "Dm[" (adb::parse-form (cadr ?x)) "]" :END))
	    (A ;; Am value
	      (list :START "Am[" (adb::parse-form (cadr ?x)) "]" :END))
	    (t ;; Should be bogus, but, hey, we're nice
	      (list :START "m[" (adb::parse-form ?x) "]" :END))))
	 (t (list :START "m[" (adb::parse-form ?x) "]" :END))))

(adb::defForm-Parser (A ?x) ;; For the raw numbers
		(list :START "A[" (adb::parse-form ?x) "]" :END))

(adb::defForm-Parser (D ?x)
		(list :START "D[" (adb::parse-form ?x) "]" :END))

(adb::defForm-Parser (set-members . ?members)
		(nconc (list :START (if (eq adb::*report-mode* :LATEX) "\\{" "{"))
			(adb::parse-argument-list ?members)
			(list (if (eq adb::*report-mode* :LATEX) "\\}" "}") :END)))

(adb::defForm-Parser (binding-list . ?elements)
		    (nconc (list :START (if (eq adb::*report-mode* :LATEX) "\\{" "{"))
			   (adb::parse-argument-list ?elements ";")
			   (list (if (eq adb::*report-mode* :LATEX) "\\}" "}") :END)))

(adb::defForm-Parser (Binding ?key . ?val)
		    (list :START (adb::parse-form ?key) ":"
			  (adb::parse-form ?val) :END))

(adb::defForm-Parser (Ordered-tuple . ?entries)
		     (nconc (list :START "<")
			    (adb::parse-argument-list ?entries)
			    (list ">" :END)))

(adb::defForm-Parser (Correspondence . ?pairs)
		     (nconc (list :START "Correspondence(")
			    (adb::parse-argument-list
			      (mapcar #'(lambda (entry) (cons 'Ordered-tuple entry))
				      ?pairs))
			    (list ")" :END)))

(adb::defForm-Parser (RCorrespondence . ?pairs)
		     (nconc (list :START "Rcorrespondence(")
			    (adb::parse-argument-list
			      (mapcar #'(lambda (entry)
					  (cond ((eq entry ':ORDERED) ':ORDERED)
						(t (cons 'Ordered-tuple entry))))
				      ?pairs))
			    (list ")" :END)))

(adb::defForm-Parser (Qprop ?a ?b)
  (if (eq adb::*report-mode* :LATEX)
      (list :START (adb::parse-form ?a) "\\( \\propto\\sb{Q+}\\  \\)" (adb::parse-form ?b))
      (list :START "Qprop(" (adb::parse-form ?a) "," (adb::parse-form ?b) ")")))

(adb::defForm-Parser (Qprop- ?a ?b)
  (if (eq adb::*report-mode* :LATEX)
      (list :START (adb::parse-form ?a) "\\( \\propto\\sb{Q-}\\  \\)" (adb::parse-form ?b))
      (list :START "Qprop-(" (adb::parse-form ?a) "," (adb::parse-form ?b) ")")))
