;;; -*- Mode: LISP; Package: QPE; Base: 10.; Syntax: Common-lisp -*-

(in-package 'QPE)

(adb::RULES-FILE "qrules")

;;; ADB rules for ATMS Qbox
;; These are stored in the :QPE-Basics packet

;;The following are always true, and must be explicitly
;;asserted for limit analysis to work.

(adb::defFact ((s (D zero)) . 0))
(adb::defFact ((s (A zero)) . 0))

(adb:rule :INTERN (((quantity ?q) . :TRUE)) (fetch-quantity ?q))

(adb:rule :INTERN (((has-function ?obj1 ?fun) . :TRUE)
		   ((has-function ?obj2 ?fun) . :TRUE))
	  (when (adb::alphalessp ?obj1 ?obj2)
	    (let ((condition (get ?fun :condition)))
	      (cond (condition
		     (nlet ((?trig1 (:EVAL (funcall condition ?obj1)))
			    (?trig2 (:EVAL (funcall condition ?obj2))))
			   (adb:rule :INTERN ((?trig1 . :TRUE)
					  (?trig2 . :TRUE))
			      (install-explicit-function ?obj1 ?obj2 ?fun))))
		    (t (install-explicit-function ?obj1 ?obj2 ?fun))))))

  ;run-time correspondence
(adb:rule :INTERN (((rcorrespondence (?n1 ?n2) . ?x) . :TRUE) :var ?f)
	  (install-runtime-correspondence (car ?f)))

(proclaim '(special **antecedents**))

(adb:rule :INTERN (((rcorrespondence :ORDERED ?pair1 ?pair2) . :TRUE) :var ?f)
   (let ((**antecedents** (cons ?f nil)))
     (make-ordered-correspondence-justifications ?pair1 ?pair2)))

(adb:rule :INTERN (((rcorrespondence
		      :ORDERED ?pair1 ?pair2 ?pair3 . ?more) . :TRUE) :var ?f)
   (error "~% Sorry, ordered correspondences between 3 or more pairs ~
not yet implemented. ~% The form in question was: ~A" ?f))        

;(adb:rule :INTERN (((has-function ?obj1 ?fun) . :TRUE)
;		   ((has-function ?obj2 ?fun) . :TRUE))
;	  (unless (or (equal ?obj1 ?obj2)
;		      (adb:alphalessp ?obj2 ?obj1))
;	    (format t "~%One pair for ~A is ~A and ~A" ?fun ?obj1 ?obj2)))

;;;;These rules prevent lossage by the user

(adb:rule :INTERN (((has-quantity ?I ?qt) . :TRUE))
	  (error  "~%You are using HAS-QUANTITY, an obsolete QPE statement.~%Please use QUANTITY instead.~%This change, though incompatible with previous QP implementations, greatly simplifies the underlying code.~%The form in question was:~% ((HAS-QUANTITY ~A ~A) . :TRUE)" ?I ?qt))

;Sorry, floating numbers do not exist any more!!!
(adb:rule :INTERN (((number ?n) . :TRUE)) 
      (error  "~%You are using NUMBER, an obsolete QPE statement.~%All numbers must now be part of a quantity.~%Furthermore, all quantities must belong to some individual.~%This change, while incompatible with previous QP implementations,  greatly simplifies the underlying code.~%The form in question was:~%((number ~A) . :TRUE)" ?n)) 

;;;; Installing inequalities
;;
;; Construct-qrel produces all the necessary assertions as a side-effect.

(adb:rule :INTERN (((less-than ?x ?y) . ?cond))
      (unless (equal ?x (car (construct-qrel ?x ?y)))
	(cerror "" "~%Bad inequality assertion: ~A < ~A" ?x ?y)))

(adb:rule :INTERN (((greater-than ?x ?y) . ?cond))
      (unless (equal ?x (car (construct-qrel ?x ?y)))
	(cerror "" "~%Bad inequality assertion: ~A > ~A" ?x ?y)))

(adb:rule :INTERN (((equal-to ?x ?y) . ?cond))
      (unless (equal ?x (car (construct-qrel ?x ?y))) 
	(cerror "" "~%Bad inequality assertion: ~A = ~A" ?x ?y)))

;;;; Implementation of steady-state assumptions

;; Many engineering analyses are only interested in steady-state
;; solutions to systems.  These operating assumptions can greatly
;; restrict the amount of effort QPE must perform.

;;; When absolutely nothing can change.  This one is pretty rare.
(adb:rule :INTERN (((total-steady-state) . :TRUE) :var ?t1
		   ((quantity (?qty . ?args)) . :TRUE) :var ?t2)
   (unless (eq (car ?args) (intern "MC" *user-package*))
     (rjustify ((s (d (?qty . ?args))) . 0) (?t1 ?t2) :TOTAL-STEADY-STATE)))

;;; Example: (steady-state-quantity (pressure container12))
(adb:rule :INTERN (((steady-state-quantity ?qty) . :TRUE) :var ?t1
		   ((quantity ?qty) . :TRUE) :var ?t2)
   (rjustify ((s (d ?qty)) . 0) (?t1 ?t2) :STEADY-QUANTITY))

;;; Example: (steady-state-individual working-fluid32)
(adb:rule :INTERN (((steady-state-individual ?ind) . :TRUE) :var ?t1
		   ((quantity (?qty ?ind)) . :TRUE) :var ?t2)
   (rjustify ((s (d (?qty ?ind))) . 0) (?t1 ?t2) :STEADY-INDIVIDUAL))


;;; Example: (steady-state-quantity-type pressure)
(adb:rule :INTERN (((steady-state-quantity-type ?qty) . :TRUE) :var ?t1
		   ((quantity (?qty . ?args)) . :TRUE) :var ?t2)

   (unless (eq (car ?args) (intern "MC" *user-package*))
     (rjustify ((s (d (?qty . ?args))) . 0) (?t1 ?t2) :STEADY-QTY-TYPE)))

