;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: QPE; -*-

;;;; Symbol & package hackery

;; At QRG we have settled on the convention that
;; all shared knowledge bases live in the USER package.   
;; Previously domain models and examples were in the QPE package.
;; This quickly became a pain when embedding QPE as a module
;; in larger systems, since each module assumed the data would be
;; in its private package.  This meant data had to be copied from
;; package to package for modules to communicate, wasting alot
;; of time and energy.

;; If you extend the QPE syntax, remember to add the newly defined symbols to
;; the list below!

(in-package 'QPE)

(eval-when (compile load eval)
 (let ((qpe-package (find-package 'QPE)))
   (dolist (sym '( ;; logical connectives & synonyms
		  -> and or iff not implies when taxonomy at-most-one <->
		  every some
		     ;; temporal scoping predicates
		     introduces uniquely-introduces there-is there-is-unique only-during
		     ;; choice sets and switches
		     choices choice-set switch settings has-setting setting
		     ;; quantities
		     A D s m quantity number greater-than less-than equal-to unrelated
		     ;; equations
		     I+ I- CTrans Qprop Qprop+ Qprop- Function-spec Correspondence Q=  = + - 
		     * / *+ /+ *0+ /0+ **rel /rel *+rel /+rel *0+rel opposite-sign Q- 
		     ordered-correspondence rcorrespondence has-function
		     ;; QPE defining forms
		     defprocess defview defentity defpredicate defperspective
		     defQuantity-Type defcomputable-relation rule defClosed-Predicate
		     ;; Set stuff
		     has-member members
		     ;; keywords in definitions
		     individuals preconditions quantityconditions quantity-conditions
		     relations influences process-instance view-instance pi vi individual-view
		     ;; Other stuff that needs to be inherited (like property-indicators)
		     individual ?self zero exists active M-D-E P-D-E
		     total-steady-state steady-state-quantity steady-state-individual
		     steady-state-quantity-type))
	   (shadowing-import (intern (format nil "~A" sym)
				     *user-package*)
			     qpe-package))))
