;; -*- Mode: Lisp; Syntax: Common-Lisp; Package: QPE -*-

;;; Timing utility

(in-package 'QPE)

;; Private copy of timer utility.
;; Should decide whether or not to package this with ADB or QPE.
;; ADB makes more sense, but have to reedit and recompile everything.

;; Internally a time will be (<run time> , <real time>)

(defun mark-time ()
  (cons (get-internal-run-time) (get-internal-real-time)))

(defun make-time (run-time real-time)
  (cons run-time real-time))

(defun time-difference (time1 time2)
  (values (- (car time1) (car time2)) (- (cdr time1) (cdr time2))))

(defun make-time-difference (time1 time2)
  (cons (- (car time1) (car time2)) (- (cdr time1) (cdr time2))))

(defun time-plus (time1 time2)
  (values (- (car time1) (car time2)) (- (cdr time1) (cdr time2))))

(defun make-time-sum (time1 time2)
  (cons (- (car time1) (car time2)) (- (cdr time1) (cdr time2))))

(defun round-to (num digits)
  (let ((mul (expt 10.0 digits)))
    (/ (round (* num mul)) mul)))

(defun convert-internal-time (time &optional (digits 3))
  (round-to (/ time internal-time-units-per-second) digits))

(defun say-time (time &key (header "") (stream *standard-output*))
  (format stream "~%  ~A~D sec. run time, ~D sec. real-time."
	  (if (string= header "") ""
	      (concatenate 'string header ": "))
	  (convert-internal-time (car time))
	  (convert-internal-time (cdr time))))

;;;; Summarizing universal time

(defun summarize-current-time (&optional (stream nil))
  (multiple-value-bind (seconds minutes hour date month year day-of-week
				daylight-savings? time-zone)
		       (get-decoded-time)
    (declare (ignore time-zone daylight-savings? day-of-week))
    (format stream "~A:~A:~A  ~A-~A-~A" hour minutes seconds
	    month date year)))

(defun extract-date (u-time)
  (multiple-value-bind (seconds minutes hour date month year day-of-week
				daylight-savings? time-zone)
		       (decode-universal-time u-time)
    (declare (ignore time-zone daylight-savings? day-of-week hour minutes seconds))
    (format nil "~A-~A-~A" month date year)))

(defun extract-time (u-time)
  (multiple-value-bind (seconds minutes hour date month year day-of-week
				daylight-savings? time-zone)
		       (decode-universal-time u-time)
    (declare (ignore time-zone daylight-savings? day-of-week year month date))
    (format nil "~A:~A:~A" hour minutes seconds)))
